@@ -4,8 +4,10 @@ from django.contrib.auth import get_user_model
 from django.contrib.auth.hashers import check_password
 from django.contrib.auth.tokens import default_token_generator
 from django.core import mail
+from django.core.urlresolvers import reverse
 from django.utils.encoding import force_bytes
 from django.utils.http import urlsafe_base64_encode
+from rest_framework import status
 
 from user_management.api import views
 from user_management.models.tests.factories import UserFactory
@@ -13,6 +15,7 @@ from user_management.models.tests.utils import APIRequestTestCase
 
 
 User = get_user_model()
+TEST_SERVER = 'http://testserver'
 
 
 class TestRegisterView(APIRequestTestCase):
@@ -31,13 +34,13 @@ class TestRegisterView(APIRequestTestCase):
         """Authenticated Users should not be able to re-register."""
         request = self.create_request('post', auth=True, data=self.data)
         response = self.view_class.as_view()(request)
-        self.assertEqual(response.status_code, 403)
+        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
 
     def test_unauthenticated_user_post(self):
         """Unauthenticated Users should be able to register."""
         request = self.create_request('post', auth=False, data=self.data)
         response = self.view_class.as_view()(request)
-        self.assertEqual(response.status_code, 201)
+        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
 
         user = User.objects.get()
         self.assertEqual(user.name, self.data['name'])
@@ -53,7 +56,7 @@ class TestRegisterView(APIRequestTestCase):
         self.data.pop('name')
         request = self.create_request('post', auth=False, data=self.data)
         response = self.view_class.as_view()(request)
-        self.assertEqual(response.status_code, 400)
+        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
 
         self.assertTrue('name' in response.data)
         self.assertFalse('password' in response.data)
@@ -64,7 +67,7 @@ class TestRegisterView(APIRequestTestCase):
         self.data['password2'] = 'something_different'
         request = self.create_request('post', auth=False, data=self.data)
         response = self.view_class.as_view()(request)
-        self.assertEqual(response.status_code, 400)
+        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
         self.assertIn('password2', response.data)
 
         self.assertFalse(User.objects.count())
@@ -80,15 +83,15 @@ class TestRegisterView(APIRequestTestCase):
 
         request = self.create_request('post', auth=False, data=self.data)
         response = self.view_class.as_view()(request)
-        self.assertEqual(response.status_code, 400)
+        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
 
         self.assertTrue('email' in response.data)
 
         self.assertFalse(User.objects.count() > 1)
 
 
-class TestPasswordResetEmailView(APIRequestTestCase):
-    view_class = views.PasswordResetEmailView
+class TestPasswordResetEmail(APIRequestTestCase):
+    view_class = views.PasswordResetEmail
 
     def test_existent_email(self):
         email = 'exists@example.com'
@@ -98,7 +101,7 @@ class TestPasswordResetEmailView(APIRequestTestCase):
         view = self.view_class.as_view()
         with patch.object(self.view_class, 'send_email') as send_email:
             response = view(request)
-        self.assertEqual(response.status_code, 204)
+        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)
 
         send_email.assert_called_once_with(user)
 
@@ -106,7 +109,7 @@ class TestPasswordResetEmailView(APIRequestTestCase):
         request = self.create_request('post', auth=True)
         view = self.view_class.as_view()
         response = view(request)
-        self.assertEqual(response.status_code, 403)
+        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
 
     def test_non_existent_email(self):
         email = 'doesnotexist@example.com'
@@ -116,7 +119,7 @@ class TestPasswordResetEmailView(APIRequestTestCase):
         view = self.view_class.as_view()
         with patch.object(self.view_class, 'send_email') as send_email:
             response = view(request)
-        self.assertEqual(response.status_code, 204)
+        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)
 
         self.assertFalse(send_email.called)
 
@@ -124,7 +127,7 @@ class TestPasswordResetEmailView(APIRequestTestCase):
         request = self.create_request('post', auth=False)
         view = self.view_class.as_view()
         response = view(request)
-        self.assertEqual(response.status_code, 400)
+        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
 
     def test_send_email(self):
         email = 'test@example.com'
@@ -144,8 +147,8 @@ class TestPasswordResetEmailView(APIRequestTestCase):
         self.assertIn('https://', sent_mail.body)
 
 
-class TestPasswordResetView(APIRequestTestCase):
-    view_class = views.PasswordResetView
+class TestPasswordReset(APIRequestTestCase):
+    view_class = views.PasswordReset
 
     def test_options(self):
         user = UserFactory.create()
@@ -155,7 +158,7 @@ class TestPasswordResetView(APIRequestTestCase):
         request = self.create_request('options', auth=False)
         view = self.view_class.as_view()
         response = view(request, uidb64=uid, token=token)
-        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
 
     def test_put(self):
         old_password = 'old_password'
@@ -174,7 +177,7 @@ class TestPasswordResetView(APIRequestTestCase):
         )
         view = self.view_class.as_view()
         response = view(request, uidb64=uid, token=token)
-        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
 
         # Get the updated user from the db
         user = User.objects.get(pk=user.pk)
@@ -198,7 +201,7 @@ class TestPasswordResetView(APIRequestTestCase):
         )
         view = self.view_class.as_view()
         response = view(request, uidb64=uid, token=token)
-        self.assertEqual(response.status_code, 400)
+        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
 
         # Get the updated user from the db
         user = User.objects.get(pk=user.pk)
@@ -211,7 +214,7 @@ class TestPasswordResetView(APIRequestTestCase):
         request = self.create_request('put', auth=False)
         view = self.view_class.as_view()
         response = view(request, uidb64=invalid_uid)
-        self.assertEqual(response.status_code, 404)
+        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
 
     def test_put_invalid_token(self):
         user = UserFactory.create()
@@ -222,11 +225,11 @@ class TestPasswordResetView(APIRequestTestCase):
         request = self.create_request('put', auth=False)
         view = self.view_class.as_view()
         response = view(request, uidb64=uid, token=token)
-        self.assertEqual(response.status_code, 404)
+        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
 
 
-class TestPasswordChangeView(APIRequestTestCase):
-    view_class = views.PasswordChangeView
+class TestPasswordChange(APIRequestTestCase):
+    view_class = views.PasswordChange
 
     def test_update(self):
         old_password = 'old_password'
@@ -246,7 +249,7 @@ class TestPasswordChangeView(APIRequestTestCase):
             })
         view = self.view_class.as_view()
         response = view(request)
-        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
 
         user = User.objects.get(pk=user.pk)
         self.assertTrue(user.check_password(new_password))
@@ -266,7 +269,7 @@ class TestPasswordChangeView(APIRequestTestCase):
         )
         view = self.view_class.as_view()
         response = view(request)
-        self.assertEqual(response.status_code, 401)
+        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
 
     def test_update_wrong_old_password(self):
         old_password = 'old_password'
@@ -286,7 +289,7 @@ class TestPasswordChangeView(APIRequestTestCase):
             })
         view = self.view_class.as_view()
         response = view(request)
-        self.assertEqual(response.status_code, 400)
+        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
 
         user = User.objects.get(pk=user.pk)
         self.assertTrue(user.check_password(old_password))
@@ -309,7 +312,7 @@ class TestPasswordChangeView(APIRequestTestCase):
             })
         view = self.view_class.as_view()
         response = view(request)
-        self.assertEqual(response.status_code, 400)
+        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
 
         user = User.objects.get(pk=user.pk)
         self.assertTrue(user.check_password(old_password))
@@ -333,7 +336,7 @@ class TestPasswordChangeView(APIRequestTestCase):
             })
         view = self.view_class.as_view()
         response = view(request)
-        self.assertEqual(response.status_code, 400)
+        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
 
         user = User.objects.get(pk=user.pk)
         self.assertTrue(user.check_password(old_password))
@@ -350,7 +353,7 @@ class TestVerifyAccountView(APIRequestTestCase):
         request = self.create_request('post', auth=True)
         view = self.view_class.as_view()
         response = view(request, uidb64=uid, token=token)
-        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
 
         updated_user = User.objects.get(pk=user.pk)
         self.assertTrue(updated_user.verified_email)
@@ -364,7 +367,7 @@ class TestVerifyAccountView(APIRequestTestCase):
         request = self.create_request('post', auth=False)
         view = self.view_class.as_view()
         response = view(request, uidb64=uid, token=token)
-        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
 
         updated_user = User.objects.get(pk=user.pk)
         self.assertTrue(updated_user.verified_email)
@@ -377,7 +380,7 @@ class TestVerifyAccountView(APIRequestTestCase):
         request = self.create_request('post')
         view = self.view_class.as_view()
         response = view(request, uidb64=invalid_uid)
-        self.assertEqual(response.status_code, 404)
+        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
 
     def test_post_invalid_token(self):
         user = UserFactory.create()
@@ -388,7 +391,7 @@ class TestVerifyAccountView(APIRequestTestCase):
         request = self.create_request('post')
         view = self.view_class.as_view()
         response = view(request, uidb64=uid, token=token)
-        self.assertEqual(response.status_code, 404)
+        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
 
     def test_post_verified_email(self):
         user = UserFactory.create(verified_email=True)
@@ -398,7 +401,7 @@ class TestVerifyAccountView(APIRequestTestCase):
         request = self.create_request('post')
         view = self.view_class.as_view()
         response = view(request, uidb64=uid, token=token)
-        self.assertEqual(response.status_code, 403)
+        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
 
     def test_post_different_user_logged_in(self):
         user = UserFactory.create()
@@ -409,7 +412,7 @@ class TestVerifyAccountView(APIRequestTestCase):
         request = self.create_request('post', user=other_user)
         view = self.view_class.as_view()
         response = view(request, uidb64=uid, token=token)
-        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
 
         updated_user = User.objects.get(pk=user.pk)
         self.assertTrue(updated_user.verified_email)
@@ -418,8 +421,8 @@ class TestVerifyAccountView(APIRequestTestCase):
         self.assertFalse(logged_in_user.verified_email)
 
 
-class TestProfileDetailView(APIRequestTestCase):
-    view_class = views.ProfileDetailView
+class TestProfileDetail(APIRequestTestCase):
+    view_class = views.ProfileDetail
 
     def expected_data(self, user):
         expected = {
@@ -435,7 +438,7 @@ class TestProfileDetailView(APIRequestTestCase):
         request = self.create_request(user=user)
         view = self.view_class.as_view()
         response = view(request)
-        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
 
         expected = self.expected_data(user)
         self.assertEqual(response.data, expected)
@@ -444,7 +447,7 @@ class TestProfileDetailView(APIRequestTestCase):
         request = self.create_request(auth=False)
         view = self.view_class.as_view()
         response = view(request)
-        self.assertEqual(response.status_code, 401)
+        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
 
     def test_put(self):
         user = UserFactory.create()
@@ -455,7 +458,7 @@ class TestProfileDetailView(APIRequestTestCase):
         request = self.create_request('put', user=user, data=data)
         view = self.view_class.as_view()
         response = view(request)
-        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
 
         expected = {}
         expected.update(self.expected_data(user))
@@ -472,7 +475,7 @@ class TestProfileDetailView(APIRequestTestCase):
         request = self.create_request('patch', user=user, data=data)
         view = self.view_class.as_view()
         response = view(request)
-        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
 
         expected = {}
         expected.update(self.expected_data(user))
@@ -484,27 +487,35 @@ class TestProfileDetailView(APIRequestTestCase):
         request = self.create_request('options')
         view = self.view_class.as_view()
         response = view(request)
-        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
 
 
-class TestProfileListView(APIRequestTestCase):
-    view_class = views.ProfileListView
+class TestUserList(APIRequestTestCase):
+    view_class = views.UserList
 
     def expected_data(self, user):
+        url = url = reverse('user_management_api:user_detail', kwargs={'pk': user.pk})
         expected = {
+            'url': TEST_SERVER + url,
             'name': user.name,
             'email': user.email,
             'date_joined': user.date_joined,
         }
         return expected
 
+    def test_get_anonymous(self):
+        request = self.create_request(auth=False)
+        view = self.view_class.as_view()
+        response = view(request)
+        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
+
     def test_get(self):
         user = UserFactory.create()
 
         request = self.create_request(user=user)
         view = self.view_class.as_view()
         response = view(request)
-        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
 
         expected = [self.expected_data(user)]
         self.assertEqual(response.data, expected)
@@ -513,4 +524,131 @@ class TestProfileListView(APIRequestTestCase):
         request = self.create_request('options')
         view = self.view_class.as_view()
         response = view(request)
-        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+
+    def test_post(self):
+        """Users should be able to create."""
+        data = {
+            'name': "Robert'); DROP TABLE Students;--'",
+            'email': 'bobby.tables+327@xkcd.com',
+        }
+        request = self.create_request('post', auth=True, data=data)
+        request.user.is_staff = True
+        response = self.view_class.as_view()(request)
+        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
+
+        user = User.objects.get(email=data['email'])
+        self.assertEqual(user.name, data['name'])
+        self.assertEqual(user.email, data['email'])
+
+    def test_post_unauthorised(self):
+        request = self.create_request('post', auth=True)
+        view = self.view_class.as_view()
+        response = view(request)
+        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
+
+
+class TestUserDetail(APIRequestTestCase):
+    view_class = views.UserDetail
+
+    def setUp(self):
+        self.user, self.other_user = UserFactory.create_batch(2)
+
+    def expected_data(self, user):
+        url = url = reverse('user_management_api:user_detail', kwargs={'pk': user.pk})
+        expected = {
+            'url': TEST_SERVER + url,
+            'name': user.name,
+            'email': user.email,
+            'date_joined': user.date_joined,
+        }
+        return expected
+
+    def check_method_forbidden(self, method):
+        request = self.create_request(method, user=self.user)
+        view = self.view_class.as_view()
+        response = view(request, pk=self.other_user.pk)
+        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
+
+    def test_get(self):
+        request = self.create_request(user=self.user)
+        view = self.view_class.as_view()
+        response = view(request, pk=self.other_user.pk)
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+
+        expected = self.expected_data(self.other_user)
+        self.assertEqual(response.data, expected)
+
+    def test_get_anonymous(self):
+        request = self.create_request(auth=False)
+        view = self.view_class.as_view()
+        response = view(request, pk=self.other_user.pk)
+        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
+
+    def test_post_unauthorised(self):
+        self.check_method_forbidden('post')
+
+    def test_put_unauthorised(self):
+        self.check_method_forbidden('put')
+
+    def test_patch_unauthorised(self):
+        self.check_method_forbidden('patch')
+
+    def test_delete_unauthorised(self):
+        self.check_method_forbidden('delete')
+
+    def test_put(self):
+        """ Tests PUT existing user for staff """
+        self.user.is_staff = True
+
+        data = {
+            'name': 'Jean Dujardin',
+        }
+
+        request = self.create_request(
+            'put',
+            user=self.user,
+            data=data)
+
+        view = self.view_class.as_view()
+
+        response = view(request, pk=self.other_user.pk)
+        self.assertEqual(response.status_code, status.HTTP_200_OK, response.data)
+
+        user = User.objects.get(pk=self.other_user.pk)
+        self.assertEqual(user.name, data['name'])
+
+    def test_patch(self):
+        """ Tests PATCH new user for staff """
+        self.user.is_staff = True
+
+        data = {
+            'name': 'Jean Deschamps',
+        }
+
+        request = self.create_request(
+            'patch',
+            user=self.user,
+            data=data)
+
+        view = self.view_class.as_view()
+
+        response = view(request, pk=self.other_user.pk)
+        self.assertEqual(response.status_code, status.HTTP_200_OK, response.data)
+
+        user = User.objects.get(pk=self.other_user.pk)
+        self.assertEqual(user.name, data['name'])
+
+    def test_delete(self):
+        """ Tests DELETE user for staff """
+        self.user.is_staff = True
+
+        request = self.create_request('delete', user=self.user)
+
+        view = self.view_class.as_view()
+
+        response = view(request, pk=self.other_user.pk)
+        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)
+
+        user = User.objects.get()
+        self.assertEqual(self.user, user)
