@@ -39,7 +39,7 @@ class UserRegister(generics.CreateAPIView):
         )
 
 
-class PasswordResetEmailView(views.APIView):
+class PasswordResetEmail(views.APIView):
     permission_classes = [permissions.IsNotAuthenticated]
 
     def post(self, request, *args, **kwargs):
@@ -93,7 +93,7 @@ class OneTimeUseAPIMixin(object):
         return super(OneTimeUseAPIMixin, self).dispatch(request, *args, **kwargs)
 
 
-class PasswordResetView(OneTimeUseAPIMixin, generics.UpdateAPIView):
+class PasswordReset(OneTimeUseAPIMixin, generics.UpdateAPIView):
     permission_classes = [permissions.IsNotAuthenticated]
     model = User
     serializer_class = serializers.PasswordResetSerializer
@@ -102,7 +102,7 @@ class PasswordResetView(OneTimeUseAPIMixin, generics.UpdateAPIView):
         return self.user
 
 
-class PasswordChangeView(generics.UpdateAPIView):
+class PasswordChange(generics.UpdateAPIView):
     model = User
     permission_classes = (IsAuthenticated,)
     serializer_class = serializers.PasswordChangeSerializer
@@ -124,7 +124,7 @@ class VerifyAccountView(OneTimeUseAPIMixin, views.APIView):
         return response.Response(status=status.HTTP_200_OK)
 
 
-class ProfileDetailView(generics.RetrieveUpdateAPIView):
+class ProfileDetail(generics.RetrieveUpdateAPIView):
     model = User
     permission_classes = (IsAuthenticated,)
     serializer_class = serializers.ProfileSerializer
@@ -133,7 +133,13 @@ class ProfileDetailView(generics.RetrieveUpdateAPIView):
         return self.request.user
 
 
-class ProfileListView(generics.ListAPIView):
+class UserList(generics.ListCreateAPIView):
     model = User
-    permission_classes = (IsAuthenticated,)
-    serializer_class = serializers.ProfileSerializer
+    permission_classes = (IsAuthenticated, permissions.IsAdminOrReadOnly)
+    serializer_class = serializers.UserSerializerCreate
+
+
+class UserDetail(generics.RetrieveUpdateDestroyAPIView):
+    model = User
+    permission_classes = (IsAuthenticated, permissions.IsAdminOrReadOnly)
+    serializer_class = serializers.UserSerializer
